@@ -1,3 +1,55 @@
+# Internal helper for categorical colors (Set1 palette, recycled)
+categorical_colors <- function(n) {
+  set1 <- c(
+    "#e41a1c", "#377eb8", "#4daf4a", "#984ea3", "#ff7f00",
+    "#ffff33", "#a65628", "#f781bf", "#999999"
+  )
+  rep_len(set1, n)
+}
+
+# Internal helper to format values with K/M notation for large numbers
+format_legend_value <- function(x) {
+
+  sapply(x, function(val) {
+    if (is.na(val)) return(NA_character_)
+    abs_val <- abs(val)
+    if (abs_val >= 1e6) {
+      paste0(round(val / 1e6, 1), "M")
+    } else if (abs_val >= 1e3) {
+      paste0(round(val / 1e3, 1), "K")
+    } else if (abs_val >= 1) {
+      as.character(round(val, 1))
+    } else {
+      as.character(round(val, 2))
+    }
+  })
+}
+
+# Internal helper to create a classification object for quickview quantile legends
+create_quickview_classification <- function(breaks, colors) {
+  n_breaks <- length(breaks) - 1
+  formatted <- format_legend_value(breaks)
+  labels <- c(
+    paste0("< ", formatted[2]),
+    if (n_breaks > 1) {
+      sapply(2:n_breaks, function(i) {
+        paste0(formatted[i], " - ", formatted[i + 1])
+      })
+    } else {
+      NULL
+    }
+  )
+  result <- list(
+    labels = labels,
+    colors = colors,
+    breaks = as.numeric(breaks),
+    method = "quantile",
+    n_classes = n_breaks
+  )
+  class(result) <- "mapgl_classification"
+  result
+}
+
 #' Quick visualization of geometries with Mapbox GL
 #'
 #' This function provides a quick way to visualize sf geometries and raster data using Mapbox GL JS.
@@ -12,6 +64,9 @@
 #' @param layer_id The layer ID to use for the visualization. Defaults to "quickview".
 #' @param legend Logical, whether to add a legend when a column is specified. Defaults to TRUE.
 #' @param legend_position The position of the legend on the map. Defaults to "top-left".
+#' @param interactive_legend Logical, whether to make the legend interactive.
+#'   When TRUE, categorical legends allow clicking to toggle visibility,
+#'   and continuous legends show a range slider. Defaults to FALSE.
 #' @param ... Additional arguments passed to mapboxgl()
 #'
 #' @return A Mapbox GL map object
@@ -46,6 +101,7 @@ mapboxgl_view <- function(
   layer_id = "quickview",
   legend = TRUE,
   legend_position = "top-left",
+  interactive_legend = FALSE,
   ...
 ) {
   if (!inherits(data, c("sf", "SpatRaster", "RasterLayer"))) {
@@ -138,11 +194,12 @@ mapboxgl_view <- function(
       map <- map |>
         add_legend(
           legend_title = if (!is.null(column)) column else "Values",
-          values = c(round(min_val, 2), round(max_val, 2)),
+          values = breaks,
           colors = legend_colors,
           type = "continuous",
           position = legend_position,
-          layer_id = layer_id
+          layer_id = layer_id,
+          interactive = interactive_legend
         )
     }
 
@@ -224,26 +281,16 @@ mapboxgl_view <- function(
             )
 
           if (legend) {
+            classification <- create_quickview_classification(breaks, colors)
             map <- map |>
               add_legend(
                 legend_title = column,
-                values = c(
-                  paste0("< ", round(breaks[2], 2)),
-                  if (n_breaks > 1) {
-                    sapply(2:n_breaks, function(i) {
-                      paste0(
-                        round(breaks[i], 2),
-                        " - ",
-                        round(breaks[i + 1], 2)
-                      )
-                    })
-                  } else NULL
-                ),
-                colors = colors,
                 type = "categorical",
                 patch_shape = "circle",
                 position = legend_position,
-                layer_id = layer_id
+                layer_id = layer_id,
+                interactive = interactive_legend,
+                classification = classification
               )
           }
         } else {
@@ -271,11 +318,12 @@ mapboxgl_view <- function(
             map <- map |>
               add_legend(
                 legend_title = column,
-                values = c(round(min_val, 2), round(max_val, 2)),
+                values = breaks,
                 colors = colors,
                 type = "continuous",
                 position = legend_position,
-                layer_id = layer_id
+                layer_id = layer_id,
+                interactive = interactive_legend
               )
           }
         }
@@ -287,7 +335,7 @@ mapboxgl_view <- function(
         }
         unique_vals <- unique(col_data[!is.na(col_data)])
         n_cats <- length(unique_vals)
-        colors <- palette(n_cats)
+        colors <- categorical_colors(n_cats)
 
         map <- map |>
           add_circle_layer(
@@ -313,7 +361,8 @@ mapboxgl_view <- function(
               type = "categorical",
               patch_shape = "circle",
               position = legend_position,
-              layer_id = layer_id
+              layer_id = layer_id,
+              interactive = interactive_legend
             )
         }
       }
@@ -376,26 +425,16 @@ mapboxgl_view <- function(
             )
 
           if (legend) {
+            classification <- create_quickview_classification(breaks, colors)
             map <- map |>
               add_legend(
                 legend_title = column,
-                values = c(
-                  paste0("< ", round(breaks[2], 2)),
-                  if (n_breaks > 1) {
-                    sapply(2:n_breaks, function(i) {
-                      paste0(
-                        round(breaks[i], 2),
-                        " - ",
-                        round(breaks[i + 1], 2)
-                      )
-                    })
-                  } else NULL
-                ),
-                colors = colors,
                 type = "categorical",
                 patch_shape = "line",
                 position = legend_position,
-                layer_id = layer_id
+                layer_id = layer_id,
+                interactive = interactive_legend,
+                classification = classification
               )
           }
         } else {
@@ -423,11 +462,12 @@ mapboxgl_view <- function(
             map <- map |>
               add_legend(
                 legend_title = column,
-                values = c(round(min_val, 2), round(max_val, 2)),
+                values = breaks,
                 colors = colors,
                 type = "continuous",
                 position = legend_position,
-                layer_id = layer_id
+                layer_id = layer_id,
+                interactive = interactive_legend
               )
           }
         }
@@ -439,7 +479,7 @@ mapboxgl_view <- function(
         }
         unique_vals <- unique(col_data[!is.na(col_data)])
         n_cats <- length(unique_vals)
-        colors <- palette(n_cats)
+        colors <- categorical_colors(n_cats)
 
         map <- map |>
           add_line_layer(
@@ -465,7 +505,8 @@ mapboxgl_view <- function(
               type = "categorical",
               patch_shape = "line",
               position = legend_position,
-              layer_id = layer_id
+              layer_id = layer_id,
+              interactive = interactive_legend
             )
         }
       }
@@ -528,25 +569,15 @@ mapboxgl_view <- function(
             )
 
           if (legend) {
+            classification <- create_quickview_classification(breaks, colors)
             map <- map |>
               add_legend(
                 legend_title = column,
-                values = c(
-                  paste0("< ", round(breaks[2], 2)),
-                  if (n_breaks > 1) {
-                    sapply(2:n_breaks, function(i) {
-                      paste0(
-                        round(breaks[i], 2),
-                        " - ",
-                        round(breaks[i + 1], 2)
-                      )
-                    })
-                  } else NULL
-                ),
-                colors = colors,
                 type = "categorical",
                 position = legend_position,
-                layer_id = layer_id
+                layer_id = layer_id,
+                interactive = interactive_legend,
+                classification = classification
               )
           }
         } else {
@@ -574,11 +605,12 @@ mapboxgl_view <- function(
             map <- map |>
               add_legend(
                 legend_title = column,
-                values = c(round(min_val, 2), round(max_val, 2)),
+                values = breaks,
                 colors = colors,
                 type = "continuous",
                 position = legend_position,
-                layer_id = layer_id
+                layer_id = layer_id,
+                interactive = interactive_legend
               )
           }
         }
@@ -590,7 +622,7 @@ mapboxgl_view <- function(
         }
         unique_vals <- unique(col_data[!is.na(col_data)])
         n_cats <- length(unique_vals)
-        colors <- palette(n_cats)
+        colors <- categorical_colors(n_cats)
 
         map <- map |>
           add_fill_layer(
@@ -615,7 +647,8 @@ mapboxgl_view <- function(
               colors = colors,
               type = "categorical",
               position = legend_position,
-              layer_id = layer_id
+              layer_id = layer_id,
+              interactive = interactive_legend
             )
         }
       }
@@ -639,6 +672,9 @@ mapboxgl_view <- function(
 #' @param layer_id The layer ID to use for the visualization. Defaults to "quickview".
 #' @param legend Logical, whether to add a legend when a column is specified. Defaults to TRUE.
 #' @param legend_position The position of the legend on the map. Defaults to "top-left".
+#' @param interactive_legend Logical, whether to make the legend interactive.
+#'   When TRUE, categorical legends allow clicking to toggle visibility,
+#'   and continuous legends show a range slider. Defaults to FALSE.
 #' @param ... Additional arguments passed to maplibre()
 #'
 #' @return A MapLibre GL map object
@@ -673,6 +709,7 @@ maplibre_view <- function(
   layer_id = "quickview",
   legend = TRUE,
   legend_position = "top-left",
+  interactive_legend = FALSE,
   ...
 ) {
   if (!inherits(data, c("sf", "SpatRaster", "RasterLayer"))) {
@@ -767,10 +804,12 @@ maplibre_view <- function(
       map <- map |>
         add_legend(
           legend_title = if (!is.null(column)) column else "Values",
-          values = c(round(min_val, 2), round(max_val, 2)),
+          values = breaks,
           colors = legend_colors,
           type = "continuous",
-          layer_id = layer_id
+          position = legend_position,
+          layer_id = layer_id,
+          interactive = interactive_legend
         )
     }
 
@@ -852,26 +891,16 @@ maplibre_view <- function(
             )
 
           if (legend) {
+            classification <- create_quickview_classification(breaks, colors)
             map <- map |>
               add_legend(
                 legend_title = column,
-                values = c(
-                  paste0("< ", round(breaks[2], 2)),
-                  if (n_breaks > 1) {
-                    sapply(2:n_breaks, function(i) {
-                      paste0(
-                        round(breaks[i], 2),
-                        " - ",
-                        round(breaks[i + 1], 2)
-                      )
-                    })
-                  } else NULL
-                ),
-                colors = colors,
                 type = "categorical",
                 patch_shape = "circle",
                 position = legend_position,
-                layer_id = layer_id
+                layer_id = layer_id,
+                interactive = interactive_legend,
+                classification = classification
               )
           }
         } else {
@@ -899,11 +928,12 @@ maplibre_view <- function(
             map <- map |>
               add_legend(
                 legend_title = column,
-                values = c(round(min_val, 2), round(max_val, 2)),
+                values = breaks,
                 colors = colors,
                 type = "continuous",
                 position = legend_position,
-                layer_id = layer_id
+                layer_id = layer_id,
+                interactive = interactive_legend
               )
           }
         }
@@ -915,7 +945,7 @@ maplibre_view <- function(
         }
         unique_vals <- unique(col_data[!is.na(col_data)])
         n_cats <- length(unique_vals)
-        colors <- palette(n_cats)
+        colors <- categorical_colors(n_cats)
 
         map <- map |>
           add_circle_layer(
@@ -941,7 +971,8 @@ maplibre_view <- function(
               type = "categorical",
               patch_shape = "circle",
               position = legend_position,
-              layer_id = layer_id
+              layer_id = layer_id,
+              interactive = interactive_legend
             )
         }
       }
@@ -1004,26 +1035,16 @@ maplibre_view <- function(
             )
 
           if (legend) {
+            classification <- create_quickview_classification(breaks, colors)
             map <- map |>
               add_legend(
                 legend_title = column,
-                values = c(
-                  paste0("< ", round(breaks[2], 2)),
-                  if (n_breaks > 1) {
-                    sapply(2:n_breaks, function(i) {
-                      paste0(
-                        round(breaks[i], 2),
-                        " - ",
-                        round(breaks[i + 1], 2)
-                      )
-                    })
-                  } else NULL
-                ),
-                colors = colors,
                 type = "categorical",
                 patch_shape = "line",
                 position = legend_position,
-                layer_id = layer_id
+                layer_id = layer_id,
+                interactive = interactive_legend,
+                classification = classification
               )
           }
         } else {
@@ -1051,11 +1072,12 @@ maplibre_view <- function(
             map <- map |>
               add_legend(
                 legend_title = column,
-                values = c(round(min_val, 2), round(max_val, 2)),
+                values = breaks,
                 colors = colors,
                 type = "continuous",
                 position = legend_position,
-                layer_id = layer_id
+                layer_id = layer_id,
+                interactive = interactive_legend
               )
           }
         }
@@ -1067,7 +1089,7 @@ maplibre_view <- function(
         }
         unique_vals <- unique(col_data[!is.na(col_data)])
         n_cats <- length(unique_vals)
-        colors <- palette(n_cats)
+        colors <- categorical_colors(n_cats)
 
         map <- map |>
           add_line_layer(
@@ -1093,7 +1115,8 @@ maplibre_view <- function(
               type = "categorical",
               patch_shape = "line",
               position = legend_position,
-              layer_id = layer_id
+              layer_id = layer_id,
+              interactive = interactive_legend
             )
         }
       }
@@ -1156,25 +1179,15 @@ maplibre_view <- function(
             )
 
           if (legend) {
+            classification <- create_quickview_classification(breaks, colors)
             map <- map |>
               add_legend(
                 legend_title = column,
-                values = c(
-                  paste0("< ", round(breaks[2], 2)),
-                  if (n_breaks > 1) {
-                    sapply(2:n_breaks, function(i) {
-                      paste0(
-                        round(breaks[i], 2),
-                        " - ",
-                        round(breaks[i + 1], 2)
-                      )
-                    })
-                  } else NULL
-                ),
-                colors = colors,
                 type = "categorical",
                 position = legend_position,
-                layer_id = layer_id
+                layer_id = layer_id,
+                interactive = interactive_legend,
+                classification = classification
               )
           }
         } else {
@@ -1202,11 +1215,12 @@ maplibre_view <- function(
             map <- map |>
               add_legend(
                 legend_title = column,
-                values = c(round(min_val, 2), round(max_val, 2)),
+                values = breaks,
                 colors = colors,
                 type = "continuous",
                 position = legend_position,
-                layer_id = layer_id
+                layer_id = layer_id,
+                interactive = interactive_legend
               )
           }
         }
@@ -1218,7 +1232,7 @@ maplibre_view <- function(
         }
         unique_vals <- unique(col_data[!is.na(col_data)])
         n_cats <- length(unique_vals)
-        colors <- palette(n_cats)
+        colors <- categorical_colors(n_cats)
 
         map <- map |>
           add_fill_layer(
@@ -1243,7 +1257,8 @@ maplibre_view <- function(
               colors = colors,
               type = "categorical",
               position = legend_position,
-              layer_id = layer_id
+              layer_id = layer_id,
+              interactive = interactive_legend
             )
         }
       }
@@ -1550,7 +1565,7 @@ add_view <- function(
         }
         unique_vals <- unique(col_data[!is.na(col_data)])
         n_cats <- length(unique_vals)
-        colors <- palette(n_cats)
+        colors <- categorical_colors(n_cats)
 
         map <- map |>
           add_circle_layer(
@@ -1705,7 +1720,7 @@ add_view <- function(
         }
         unique_vals <- unique(col_data[!is.na(col_data)])
         n_cats <- length(unique_vals)
-        colors <- palette(n_cats)
+        colors <- categorical_colors(n_cats)
 
         map <- map |>
           add_line_layer(
@@ -1859,7 +1874,7 @@ add_view <- function(
         }
         unique_vals <- unique(col_data[!is.na(col_data)])
         n_cats <- length(unique_vals)
-        colors <- palette(n_cats)
+        colors <- categorical_colors(n_cats)
 
         map <- map |>
           add_fill_layer(
